@@ -17,7 +17,9 @@ where
     K: AsRef<str>,
 {
     pub fn new() -> Self {
-        Self { entries: Default::default() }
+        Self {
+            entries: Default::default(),
+        }
     }
 
     pub fn iter(&self) -> impl Iterator<Item = (&K, &V)> {
@@ -29,15 +31,22 @@ where
     }
 
     pub fn contains_key<Q: AsRef<str>>(&self, key: Q) -> bool {
-        self.iter().any(|e| e.0.as_ref().eq_ignore_ascii_case(key.as_ref()))
+        self.iter()
+            .any(|e| e.0.as_ref().eq_ignore_ascii_case(key.as_ref()))
     }
 
     pub fn get<Q: AsRef<str>>(&self, key: Q) -> Option<&V> {
-        self.iter().find(|e| e.0.as_ref().eq_ignore_ascii_case(key.as_ref())).map(|e| e.1)
+        self.iter()
+            .find(|e| e.0.as_ref().eq_ignore_ascii_case(key.as_ref()))
+            .map(|e| e.1)
     }
 
     pub fn insert(&mut self, key: K, value: V) -> Option<V> {
-        match self.entries.iter_mut().find(|e| e.0.as_ref().eq_ignore_ascii_case(key.as_ref())) {
+        match self
+            .entries
+            .iter_mut()
+            .find(|e| e.0.as_ref().eq_ignore_ascii_case(key.as_ref()))
+        {
             None => {
                 self.entries.push((key, value));
                 None
@@ -65,23 +74,25 @@ impl<V> StrVecMap<String, V> {
 
 /// A vector set containing ASCII-case-insensitive `AsRef<str>` elements.
 #[derive(Clone, Debug, Default)]
-pub struct StrVecSet<E> {
-    map: StrVecMap<E, ()>,
+pub struct StrVecSet<T> {
+    map: StrVecMap<T, ()>,
 }
 
-impl<E> StrVecSet<E>
+impl<T> StrVecSet<T>
 where
-    E: AsRef<str>,
+    T: AsRef<str>,
 {
     pub fn new() -> Self {
-        Self { map: StrVecMap::new() }
+        Self {
+            map: StrVecMap::new(),
+        }
     }
 
     pub fn contains<Q: AsRef<str>>(&self, key: Q) -> bool {
         self.map.contains_key(key)
     }
 
-    pub fn insert(&mut self, key: E) -> bool {
+    pub fn insert(&mut self, key: T) -> bool {
         self.map.insert(key, ()).is_none()
     }
 }
