@@ -3,20 +3,21 @@ use crate::{
     config::Config,
     error::{Error, Result},
 };
-use milter::ActionContext;
+use indymilter::ContextActions;
 use once_cell::sync::Lazy;
 use std::{
+    ffi::CString,
     fmt::{self, Display, Formatter},
     str,
 };
 
-#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
 pub struct Header<'a> {
     pub name: &'a str,
     pub value: &'a str,
 }
 
-#[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+#[derive(Clone, Debug, Eq, Hash, PartialEq)]
 pub struct Email<'a> {
     pub header: Vec<Header<'a>>,
     pub body: &'a [u8],
@@ -74,7 +75,7 @@ fn header_lines(header: &[u8]) -> Vec<&[u8]> {
 
 fn parse_header_line(bytes: &[u8]) -> Result<Header<'_>> {
     // This assumes that headers received back from SpamAssassin are valid
-    // UTF-8, which is plausible since the client only sent UTF-8 earlier.
+    // UTF-8, which should be the case since the client only sent UTF-8 earlier.
     let line = str::from_utf8(bytes).map_err(|_| Error::ParseEmail)?;
 
     let (name, value) = line.split_at(line.find(':').ok_or(Error::ParseEmail)?);
@@ -107,7 +108,7 @@ pub fn is_spam_assassin_header(name: &str) -> bool {
 
 // Values use CRLF line breaks and include leading whitespace.
 pub type HeaderMap = StrVecMap<String, String>;
-pub type HeaderSet<'e> = StrVecSet<&'e str>;
+pub type HeaderSet<'a> = StrVecSet<&'a str>;
 
 pub static REWRITE_HEADERS: Lazy<HeaderSet<'static>> = Lazy::new(|| {
     let mut h = HeaderSet::new();
@@ -230,51 +231,54 @@ impl<'a, 'c> HeaderRewriter<'a, 'c> {
         self.spam_flag.unwrap_or(false)
     }
 
-    pub fn rewrite_spam_assassin_headers(
+    pub async fn rewrite_spam_assassin_headers(
         &self,
         id: &str,
-        actions: &impl ActionContext,
-    ) -> milter::Result<()> {
+        actions: &impl ContextActions,
+    ) -> Result<()> {
+        let mods = self.spam_assassin_mods.iter();
         if self.prepend.unwrap_or(false) {
             // Prepend ‘X-Spam-’ headers in reverse order, so that they appear
             // in the order received from SpamAssassin.
-            execute_mods(id, self.spam_assassin_mods.iter().rev(), actions, self.config)?;
+            execute_mods(id, mods.rev(), actions, self.config).await?;
         } else {
-            execute_mods(id, self.spam_assassin_mods.iter(), actions, self.config)?;
+            execute_mods(id, mods, actions, self.config).await?;
         }
 
         // Delete all incoming ‘X-Spam-’ headers not returned by SpamAssassin to
         // get rid of foreign ‘X-Spam-Flag’ etc. headers.
-        let deletions = self.original.keys()
+        let deletions = self
+            .original
+            .keys()
             .filter(|n| is_spam_assassin_header(n) && !self.processed.contains(n))
             .map(|name| HeaderMod::Delete { name })
             .collect::<Vec<_>>();
-        execute_mods(id, deletions.iter(), actions, self.config)
+        execute_mods(id, deletions.iter(), actions, self.config).await
     }
 
-    pub fn rewrite_rewrite_headers(
+    pub async fn rewrite_rewrite_headers(
         &self,
         id: &str,
-        actions: &impl ActionContext,
-    ) -> milter::Result<()> {
-        execute_mods(id, self.rewrite_mods.iter(), actions, self.config)
+        actions: &impl ContextActions,
+    ) -> Result<()> {
+        execute_mods(id, self.rewrite_mods.iter(), actions, self.config).await
     }
 
-    pub fn rewrite_report_headers(
+    pub async fn rewrite_report_headers(
         &self,
         id: &str,
-        actions: &impl ActionContext,
-    ) -> milter::Result<()> {
-        execute_mods(id, self.report_mods.iter(), actions, self.config)
+        actions: &impl ContextActions,
+    ) -> Result<()> {
+        execute_mods(id, self.report_mods.iter(), actions, self.config).await
     }
 }
 
-fn execute_mods<'a, I>(
+async fn execute_mods<'a, I>(
     id: &str,
     mods: I,
-    actions: &impl ActionContext,
+    actions: &impl ContextActions,
     config: &Config,
-) -> milter::Result<()>
+) -> Result<()>
 where
     I: IntoIterator<Item = &'a HeaderMod<'a>>,
 {
@@ -283,30 +287,30 @@ where
             verbose!(config, "{}: rewriting header: {} [dry run, not done]", id, m);
         } else {
             verbose!(config, "{}: rewriting header: {}", id, m);
-            m.execute(actions)?;
+            m.execute(actions).await?;
         }
     }
     Ok(())
 }
 
-pub fn replace_body(
+pub async fn replace_body(
     id: &str,
     body: &[u8],
-    actions: &impl ActionContext,
+    actions: &impl ContextActions,
     config: &Config,
-) -> milter::Result<()> {
+) -> Result<()> {
     if config.dry_run() {
         verbose!(config, "{}: replacing message body [dry run, not done]", id);
     } else {
         verbose!(config, "{}: replacing message body", id);
-        actions.append_body_chunk(body)?;
+        actions.replace_body(body).await?;
     }
     Ok(())
 }
 
 /// A header rewriting modification operation. These are intended to operate
 /// only on the first instance of headers occurring multiple times.
-#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
 enum HeaderMod<'a> {
     Add { name: &'a str, value: &'a str, prepend: bool },
     Replace { name: &'a str, value: &'a str, prepend: bool },
@@ -315,39 +319,41 @@ enum HeaderMod<'a> {
 }
 
 impl HeaderMod<'_> {
-    fn execute(&self, actions: &impl ActionContext) -> milter::Result<()> {
-        use HeaderMod::*;
-
+    async fn execute(&self, actions: &impl ContextActions) -> Result<()> {
         // The milter library is smart enough to treat the name in a
         // case-insensitive manner, eg ‘Subject’ may replace ‘sUbject’.
         match *self {
-            Add { name, value, prepend } => add_header(actions, name, value, prepend),
-            Replace { name, value, prepend } => {
-                delete_header(actions, name)?;
-                add_header(actions, name, value, prepend)?;
-                Ok(())
+            Self::Add { name, value, prepend } => add_header(actions, name, value, prepend).await?,
+            Self::Replace { name, value, prepend } => {
+                delete_header(actions, name).await?;
+                add_header(actions, name, value, prepend).await?;
             }
-            Modify { name, value } => actions.replace_header(name, 1, Some(&ensure_lf(value))),
-            Delete { name } => delete_header(actions, name),
+            Self::Modify { name, value } => {
+                actions.change_header(name, 1, Some(ensure_lf(value))).await?;
+            }
+            Self::Delete { name } => delete_header(actions, name).await?,
         }
+        Ok(())
     }
 }
 
-fn add_header(
-    actions: &impl ActionContext,
+async fn add_header(
+    actions: &impl ContextActions,
     name: &str,
     value: &str,
     prepend: bool,
-) -> milter::Result<()> {
+) -> Result<()> {
     if prepend {
-        actions.insert_header(0, name, &ensure_lf(value))
+        actions.insert_header(0, name, ensure_lf(value)).await?;
     } else {
-        actions.add_header(name, &ensure_lf(value))
+        actions.add_header(name, ensure_lf(value)).await?;
     }
+    Ok(())
 }
 
-fn delete_header(actions: &impl ActionContext, name: &str) -> milter::Result<()> {
-    actions.replace_header(name, 1, None)
+async fn delete_header(actions: &impl ContextActions, name: &str) -> Result<()> {
+    actions.change_header(name, 1, None::<CString>).await?;
+    Ok(())
 }
 
 impl Display for HeaderMod<'_> {
@@ -370,9 +376,9 @@ mod tests {
 
     #[test]
     fn email_split_at_eoh() {
-        assert_eq!(split_at_eoh(b"x\r\n\r\ny"), Ok((b"x\r\n" as &[_], b"y" as &[_])));
-        assert_eq!(split_at_eoh(b"x\r\n\r\n"), Ok((b"x\r\n" as &[_], b"" as &[_])));
-        assert_eq!(split_at_eoh(b"\r\n\r\ny"), Ok((b"\r\n" as &[_], b"y" as &[_])));
+        assert_eq!(split_at_eoh(b"x\r\n\r\ny"), Ok((&b"x\r\n"[..], &b"y"[..])));
+        assert_eq!(split_at_eoh(b"x\r\n\r\n"), Ok((&b"x\r\n"[..], &b""[..])));
+        assert_eq!(split_at_eoh(b"\r\n\r\ny"), Ok((&b"\r\n"[..], &b"y"[..])));
         assert_eq!(split_at_eoh(b"\r\ny"), Err(Error::ParseEmail));
         assert_eq!(split_at_eoh(b"y"), Err(Error::ParseEmail));
     }
@@ -380,25 +386,25 @@ mod tests {
     #[test]
     fn email_header_lines_empty() {
         assert_eq!(header_lines(b""), Vec::<&[_]>::new());
-        assert_eq!(header_lines(b"\r\n"), vec![b"" as &[_]]);
-        assert_eq!(header_lines(b"\r\n\r\n"), vec![b"" as &[_], b"" as &[_]]);
+        assert_eq!(header_lines(b"\r\n"), vec![&b""[..]]);
+        assert_eq!(header_lines(b"\r\n\r\n"), vec![&b""[..], &b""[..]]);
     }
 
     #[test]
     fn email_header_lines_simple() {
-        assert_eq!(header_lines(b"x\r\n"), vec![b"x" as &[_]]);
-        assert_eq!(header_lines(b"x\r\ny"), vec![b"x" as &[_], b"y" as &[_]]);
-        assert_eq!(header_lines(b"x\r\ny\r\n"), vec![b"x" as &[_], b"y" as &[_]]);
+        assert_eq!(header_lines(b"x\r\n"), vec![&b"x"[..]]);
+        assert_eq!(header_lines(b"x\r\ny"), vec![&b"x"[..], &b"y"[..]]);
+        assert_eq!(header_lines(b"x\r\ny\r\n"), vec![&b"x"[..], &b"y"[..]]);
     }
 
     #[test]
     fn email_header_lines_multi() {
-        assert_eq!(header_lines(b"x\r\n\t"), vec![b"x\r\n\t" as &[_]]);
-        assert_eq!(header_lines(b"x\r\n\ty"), vec![b"x\r\n\ty" as &[_]]);
-        assert_eq!(header_lines(b"x\r\n\ty\r\n"), vec![b"x\r\n\ty" as &[_]]);
+        assert_eq!(header_lines(b"x\r\n\t"), vec![&b"x\r\n\t"[..]]);
+        assert_eq!(header_lines(b"x\r\n\ty"), vec![&b"x\r\n\ty"[..]]);
+        assert_eq!(header_lines(b"x\r\n\ty\r\n"), vec![&b"x\r\n\ty"[..]]);
         assert_eq!(
             header_lines(b"x\r\n\ty\r\n\tz\r\nq"),
-            vec![b"x\r\n\ty\r\n\tz" as &[_], b"q" as &[_]]
+            vec![&b"x\r\n\ty\r\n\tz"[..], &b"q"[..]]
         );
     }
 
@@ -453,7 +459,7 @@ mod tests {
     #[test]
     fn header_rewriter_flags_spam() {
         let mut headers = HeaderMap::new();
-        headers.insert(String::from("x-spam-flag"), String::from(" no"));
+        headers.insert("x-spam-flag".into(), " no".into());
         let config = Default::default();
 
         let mut rewriter = HeaderRewriter::new(headers, &config);
@@ -486,10 +492,10 @@ mod tests {
     #[test]
     fn header_rewriter_adds_and_replaces_headers() {
         let mut headers = HeaderMap::new();
-        headers.insert(String::from("x-spam-level"), String::from(" ***"));
-        headers.insert(String::from("subject"), String::from(" original"));
-        headers.insert(String::from("x-spam-prev-subject"), String::from(" very original"));
-        headers.insert(String::from("to"), String::from(" recipient@gluet.ch"));
+        headers.insert("x-spam-level".into(), " ***".into());
+        headers.insert("subject".into(), " original".into());
+        headers.insert("x-spam-prev-subject".into(), " very original".into());
+        headers.insert("to".into(), " recipient@gluet.ch".into());
         let config = Default::default();
 
         let mut rewriter = HeaderRewriter::new(headers, &config);
