@@ -1,6 +1,6 @@
 -- ‘Happy path’ processing of an ordinary ham (not spam) message.
 
-conn = mt.connect("inet:3333@localhost")
+local conn = mt.connect("inet:" .. port .. "@127.0.0.1")
 assert(conn, "could not open connection")
 
 local err = mt.conninfo(conn, "client.gluet.ch", "123.123.123.123")
@@ -65,7 +65,7 @@ assert(mt.getreply(conn) == SMFIR_CONTINUE)
 
 local err = mt.eom(conn)
 assert(err == nil, err)
-assert(mt.getreply(conn) == SMFIR_ACCEPT)
+assert(mt.getreply(conn) == SMFIR_CONTINUE)
 
 assert(mt.eom_check(conn, MT_HDRINSERT, "X-Spam-Custom", " Custom-Value"))
 assert(mt.eom_check(conn, MT_HDRDELETE, "X-Spam-Checker-Version"))
