@@ -1,6 +1,6 @@
 -- 1) A connection from the loopback IP address is accepted.
 
-conn = mt.connect("inet:3333@localhost")
+local conn = mt.connect("inet:" .. port .. "@127.0.0.1")
 assert(conn, "could not open connection")
 
 local err = mt.conninfo(conn, nil, "127.0.0.1")
@@ -13,7 +13,7 @@ assert(err == nil, err)
 -- 2) A connection from an ‘unknown’ IP address (for example, from a UNIX
 -- domain socket) is also accepted.
 
-conn = mt.connect("inet:3333@localhost")
+local conn = mt.connect("inet:" .. port .. "@127.0.0.1")
 assert(conn, "could not open connection")
 
 local err = mt.conninfo(conn, nil, "unspec")
